
/*****************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit www.shmoo.com/osiris for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      database_handler.c
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#include "database_handler.h"
#include "debug.h"
#include "config.h"

/******************************************************************************
**
**    Function: database_handler_new
**
**    Purpose:  create a new database abstraction structure
**              with safe defaults.  here we setup the function
**              pointers into the specific database module.
**
******************************************************************************/

database_handler * database_handler_new()
{
    database_handler *db = osi_malloc( sizeof( database_handler ) );

    db->state   = DB_STATE_CLOSED;
    db->handle  = NULL;
    db->counter = 0;
    db->name    = NULL;

    /* setup the pointers into the gdbm module */
    /* if we are using it.                     */

#ifdef MODULE_GDBM

    db->open                  = module_gdbm_open;
    db->close                 = module_gdbm_close;

    db->store_record          = module_gdbm_store_record;
    db->retrieve_record       = module_gdbm_retrieve_record;

    db->exists                = module_gdbm_exists;
    db->record_exists         = module_gdbm_record_exists;

    db->store_header          = module_gdbm_store_header;
    db->retrieve_header       = module_gdbm_retrieve_header;

    db->retrieve_first_record = module_gdbm_retrieve_first_record;
    db->retrieve_next_record  = module_gdbm_retrieve_next_record;

    db->empty                 = module_gdbm_empty;
    db->retrieve_last_error   = module_gdbm_retrieve_last_error;

#endif

#ifdef MODULE_MYSQL

    db->open                  = module_mysql_open;
    db->close                 = module_mysql_close;

    db->store_record          = module_mysql_store_record;
    db->retrieve_record       = module_mysql_retrieve_record;

    db->exists                = module_mysql_exists;
    db->record_exists         = module_mysql_record_exists;

    db->store_header          = module_mysql_store_header;
    db->retrieve_header       = module_mysql_retrieve_header;

    db->retrieve_first_record = module_mysql_retrieve_first_record;
    db->retrieve_next_record  = module_mysql_retrieve_next_record;

    db->empty                 = module_mysql_empty;
    db->retrieve_last_error   = module_mysql_retrieve_last_error;
#endif

    return db;
}

/******************************************************************************
**
**    Function: database_handler_destroy
**
**    Purpose:  destroy the handler object, close database
**              in the case where it wasn't shut down.
**
******************************************************************************/

void database_handler_destroy( database_handler *db )
{
    if( db != NULL )
    {
        OSI_DEBUG( __POS__, DEBUG_DATABASE,
                  "destroying db handler: %s", db->name );

        /* in case somebody was dumb. */

        if( db->state == DB_STATE_OPEN )
        {
            db->close( db );
        }

        osi_free( db );
    }
}
