
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit www.shmoo.com/osiris for
**  details.
**
******************************************************************************/

/*****************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      database_handler.h
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#ifndef DATABASE_HANDLER
#define DATABASE_HANDLER

#include "utilities.h"
#include "record.h"

/* db open states and modes. */

#define DB_STATE_CLOSED   0
#define DB_STATE_OPEN     1
#define DB_STATE_ERROR    2

#define DB_READ           1   /* read only access.  */
#define DB_WRITE          2   /* read/write access. */
#define DB_NEW            3   /* read/write/create  */


#define HEADER_KEY "osiris_toc"   /* change this to something unique, someday */


/* basic database abstraction structure. */


typedef struct database_handler
{
    unsigned int state;
	void *handle;

	char *name;
    unsigned long counter;


    /* IMPLEMENTATION REQUIRED. */

    bool (*open)( struct database_handler *db, int mode );
    bool (*close)( struct database_handler *db );

    bool (*store_record)( struct database_handler *db, record *osi_record );
    bool (*record_exists)( struct database_handler *db, record *osi_record );

    bool (*store_header)( struct database_handler *db, char *header );
    char * (*retrieve_header)( struct database_handler *db );

    bool (*exists)( struct database_handler *db );

    bool (*retrieve_record)( struct database_handler *db, const char *path,
                             record *osi_record );

    bool (*retrieve_first_record)( struct database_handler *db,
                                   record *osi_record );

    bool (*retrieve_next_record)( struct database_handler *db,
                                  record *osi_record );

    char * (*retrieve_last_error)( struct database_handler *db );
    bool (*empty)( struct database_handler *db );

} database_handler;


/* basic abstraction functions. */

database_handler * database_handler_new();
void database_handler_destroy( database_handler *db );

/* gdbm module prototypes. */

bool module_gdbm_open( database_handler *db, int mode );
bool module_gdbm_close( database_handler *db );

bool module_gdbm_store_record( database_handler *db, record *osi_record );
bool module_gdbm_record_exists( database_handler *db, record *osi_record );
bool module_gdbm_exists( database_handler *db );

bool module_gdbm_store_header( database_handler *db, char *header );
bool module_gdbm_empty( database_handler *db );

char * module_gdbm_retrieve_header( database_handler *db );

bool module_gdbm_retrieve_record( database_handler *db, const char *path,
                                  record *osi_record );

bool module_gdbm_retrieve_first_record( database_handler *db,
                                        record *osi_record );

bool module_gdbm_retrieve_next_record( database_handler *db,
                                       record *osi_record );

char * module_gdbm_retrieve_last_error( database_handler *db );

/* mysql prototypes. */

bool module_mysql_open( database_handler *db, int mode );
bool module_mysql_close( database_handler *db );

bool module_mysql_store_record( database_handler *db, record *osi_record );
bool module_mysql_record_exists( database_handler *db, record *osi_record );
bool module_mysql_exists( database_handler *db );

bool module_mysql_store_header( database_handler *db, char *header );
bool module_mysql_empty( database_handler *db );

char * module_mysql_retrieve_header( database_handler *db );

bool module_mysql_retrieve_record( database_handler *db, const char *path,
                                   record *osi_record );

bool module_mysql_retrieve_first_record( database_handler *db,
                                         record *osi_record );

bool module_mysql_retrieve_next_record( database_handler *db,
                                        record *osi_record );

char * module_mysql_retrieve_last_error( database_handler *db );

#endif
