
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit www.shmoo.com/osiris for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      list.c
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#include "list.h"

/******************************************************************************
**
**    Function: list_new
**
**    Purpose:  create a new list structure, which consists of
**              creating a dummy none.
**
******************************************************************************/

list list_new()
{
    node *head = osi_malloc( sizeof( struct node ) );

    head->data = NULL;
    head->next = NULL;

    return head;
}

/******************************************************************************
**
**    Function: list_destroy
**
**    Purpose:  frees the nodes in the list.  careful, if you are
**              storing data, you will need to remove it by hand
**              before destroying the actual list.
**
******************************************************************************/

void list_destroy( list head )
{
    node *temp;

    if( head == NULL )
    {
        return;
    }

    while( head )
    {
        temp = head->next;
        osi_free( head );
        head = temp;
    }
}

/******************************************************************************
**
**    Function: list_add
**
**    Purpose:  add an element to the back of the list.
**
******************************************************************************/

bool list_add( list head, void *new_data )
{
    node *new_node;
    node *temp;

    if( head == NULL )
    {
        return FALSE;
    }

    new_node = osi_malloc( sizeof( struct node ) );

    new_node->data = new_data;
    new_node->next = NULL;

    /* go through list and find the last valid node. */

    temp = head;

    while( temp->next != NULL )
    {
        temp = temp->next;
    }

    /* insert our new node. */

    temp->next = new_node;

    return TRUE;
}

/******************************************************************************
**
**    Function: list_get_first_node
**
**    Purpose:  skip over dummy node and return the first data node.
**
******************************************************************************/

node * list_get_first_node( list head )
{
    /* first node is dummy, return node */
    /* after dummy node.                */

    return list_get_next_node( head );
}

/******************************************************************************
**
**    Function: list_get_next_node
**
**    Purpose:  attempt to get the node after the specified node.
**
*****************************************************************************/

list list_get_next_node( list head )
{
    if( head == NULL )
    {
        return NULL;
    }

    return head->next;
}

/******************************************************************************
**
**    Function: list_dump
**
**    Purpose:  raw list data dump, used for debugging.
**
******************************************************************************/

void list_dump( list head )
{
    list start = head;

    if( head == NULL )
    {
        return;
    }

    fprintf( stderr, " ----[ list dump ]----\n\n" );
    fprintf( stderr, " (head) " );

    head = head->next;

    while( head != NULL )
    {
        fprintf( stderr, "-> <node - data:" );

        if( head->data != NULL )
        {
            fprintf( stderr, "%d> ", head->data );
        }

        else
        {
            fprintf( stderr, "null> " );
        }

        head = head->next;

        if( head == NULL )
        {
            fprintf( stderr, "-> NULL" );
        }

        else
        {
            fprintf( stderr, "->data " );
        }
    }

    fprintf( stderr, "\n\n" );
}
