
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit osiris.shmoo.com for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      osiris.h
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#ifndef OSIRIS_H
#define OSIRIS_H

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <dirent.h>

#include <sys/time.h>
#include <sys/resource.h>

#include "configuration.h"
#include "debug.h"
#include "record.h"
#include "database_handler.h"

#define PROGRAM_NAME     "osiris"
#define PROGRAM_VERSION  "1.5.2"

#ifdef MODULE_MYSQL
#define DB_MODULE "mysql"
#endif

#ifdef MODULE_GDBM
#define DB_MODULE "gdbm"
#endif

/* common default process priority. */

#ifndef NZERO
#define NZERO 20
#endif

#define LOW_PROCESS_PRIORITY ( ( NZERO * 2 ) - 1 )

/* debug categories. */

unsigned int debug_categories = DEBUG_ALL;


/* default global runtime values. */


#define DEFAULT_GLOBAL_DIRECTORY     "."
#define DEFAULT_GLOBAL_DATABASE      ""

#define DEFAULT_GLOBAL_VERBOSE       TRUE
#define DEFAULT_GLOBAL_PROMPT        TRUE
#define DEFAULT_GLOBAL_SHOW_ERRORS   TRUE
#define DEFAULT_GLOBAL_RECURSIVE     TRUE
#define DEFAULT_GLOBAL_FOLLOW_LINKS  FALSE

#define DEFAULT_GLOBAL_HASH          HASH_MD5
#define DEFAULT_GLOBAL_ACTION        ACTION_TYPE_INCLUDE_ALL

/* default attribute list is everything except  */
/* the atime attribute, it's almost a worthless */
/* attribute to watch.                          */

#define DEFAULT_GLOBAL_ATTRIBUTES ( ATTRIBUTE_PERM  | ATTRIBUTE_MTIME |\
                                    ATTRIBUTE_INODE | ATTRIBUTE_GID |\
                                    ATTRIBUTE_LINKS | ATTRIBUTE_UID |\
                                    ATTRIBUTE_GID   | ATTRIBUTE_BYTES |\
                                    ATTRIBUTE_CTIME | ATTRIBUTE_BLOCKS |\
                                    ATTRIBUTE_FLAGS )

/* osiris global data structures. */

configuration *config_module        = NULL;

block *current_block                = NULL;
block *global_block                 = NULL;

string_list *directory_list         = NULL;
string_list *bad_directory_list     = NULL;
string_list *database_list          = NULL;
string_list *bad_database_list      = NULL;

list db_handles_list                = NULL;

char start_time[100]                = "";
char end_time[100]                  = "";

unsigned long files_logged_count           = 0;
unsigned long files_encountered_count      = 0;
unsigned long files_unreadable_count       = 0;
unsigned long directories_unreadable_count = 0;
unsigned long symlinks_followed_count      = 0;
unsigned long symlinks_encountered_count   = 0;
unsigned long database_errors_count        = 0;


/* function prototypes. */

void initialize( int argument_count, char *argument_list[] );
void halt();
void process();

void parse_arguments( int argument_count, char *argument_list[] );
void print_database_error( database_handler *db );
void print_usage();
void print_program_info();
void print_summary();

void open_config_db_files();
void close_config_db_files();

void log_file( record *file_record, struct stat *stats );

void process_block( block *config_block );
void process_directory( const char *directory_path );
void process_file( const char *file_path, struct stat *file_stats );

void initialize_global_block();

void verify_config_state();
void verify_config_file();

void expand_directory_list();
void verify_directory_list();
void create_blocks_for_directory_list();

void create_databases_for_blocks();
void destroy_database_handlers();

void store_database_headers();
void signal_handler( int signal );

bool get_configuration_file_name( int argument_count, char *argument_list[] );
block *create_default_block();


#endif
