
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit www.shmoo.com/osiris for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      record.c
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#include <sys/types.h>
#include <pwd.h>
#include <grp.h>

#include "utilities.h"
#include "record.h"


void wrap_record( record *r )
{
    if( r == NULL )
    {
        return;
    }

    r->reason = htonl( r->reason );
    r->attributes = htonl( r->attributes );

    r->stats.device = HTONLL( r->stats.device );
    r->stats.inode = HTONLL( r->stats.inode );
    r->stats.permissions = HTONLL( r->stats.permissions );
    r->stats.links = HTONLL( r->stats.links );
    r->stats.uid = HTONLL( r->stats.uid );
    r->stats.gid = HTONLL( r->stats.gid );
    r->stats.mtime = HTONLL( r->stats.mtime );
    r->stats.atime = HTONLL( r->stats.atime );
    r->stats.ctime = HTONLL( r->stats.ctime );
    r->stats.device_type = HTONLL( r->stats.device_type );
    r->stats.bytes = HTONLL( r->stats.bytes );
    r->stats.blocks = HTONLL( r->stats.blocks );
    r->stats.block_size = HTONLL( r->stats.block_size );
    r->stats.flags = HTONLL( r->stats.flags );
}

void unwrap_record( record *r )
{
    if( r == NULL )
    {
        return;
    }

    r->reason = ntohl( r->reason );
    r->attributes = ntohl( r->attributes );

    r->stats.device = NTOHLL( r->stats.device );
    r->stats.inode = NTOHLL( r->stats.inode );
    r->stats.permissions = NTOHLL( r->stats.permissions );
    r->stats.links = NTOHLL( r->stats.links );
    r->stats.uid = NTOHLL( r->stats.uid );
    r->stats.gid = NTOHLL( r->stats.gid );
    r->stats.mtime = NTOHLL( r->stats.mtime );
    r->stats.atime = NTOHLL( r->stats.atime );
    r->stats.ctime = NTOHLL( r->stats.ctime );
    r->stats.device_type = NTOHLL( r->stats.device_type );
    r->stats.bytes = NTOHLL( r->stats.bytes );
    r->stats.blocks = NTOHLL( r->stats.blocks );
    r->stats.block_size = NTOHLL( r->stats.block_size );
    r->stats.flags = NTOHLL( r->stats.flags );
}

void print_record( record *r )
{
    char perms[15]  = "";
    char user[256]  = "";
    char group[256] = "";
    char date[20]   = "";

    time_t m_time;
    time_t a_time;
    time_t c_time;

    struct passwd *file_owner;
    struct group *file_group;

    if( r == NULL )
    {
        fprintf( stdout, "<null record>\n" );
        return;
    }

    m_time = (time_t)r->stats.mtime;
    a_time = (time_t)r->stats.atime;
    c_time = (time_t)r->stats.ctime;

    file_owner = getpwuid( (uid_t)r->stats.uid );
    file_group = getgrgid( (gid_t)r->stats.gid );

    if( file_owner != NULL )
    {
        safe_strlcpy( user, file_owner->pw_name, sizeof( user ) );
    }

    if( file_group != NULL )
    {
        safe_strlcpy( group, file_group->gr_name, sizeof( group ) );
    }

    get_file_attribute_string( perms, sizeof( perms ), r->stats.permissions );

    fprintf( stdout, "====[ file record ]====\n\n" );

    fprintf( stdout, "path: (%s)\n",
             escape_filename( r->file_path ) );

    fprintf( stdout, "checksum: %s(%s)\n", r->hash_algorithm, r->hash );

    fprintf( stdout, "user:  %s\n", user );
    fprintf( stdout, "group: %s\n", group );

    fprintf( stdout, "device: %d\n", (int)r->stats.device );
    fprintf( stdout, "inode: %d\n", (int)r->stats.inode );

    get_file_attribute_string( perms, sizeof( perms ), r->stats.permissions );
    fprintf( stdout, "permissions: %s\n", perms );

    fprintf( stdout, "links: %d\n", (int)r->stats.links );
    fprintf( stdout, "uid: %d\n", (int)r->stats.uid );
    fprintf( stdout, "gid: %d\n", (int)r->stats.gid );

    fprintf( stdout, "mtime: %s", ctime( &m_time ) );
    fprintf( stdout, "atime: %s", ctime( &a_time ) );
    fprintf( stdout, "ctime: %s", ctime( &c_time ) );

    fprintf( stdout, "device_type: %d\n", (int)r->stats.device_type );
    fprintf( stdout, "bytes: %d\n", (int)r->stats.bytes );
    fprintf( stdout, "blocks: %d\n", (int)r->stats.blocks );
    fprintf( stdout, "block_size: %d\n", (int)r->stats.block_size );
    fprintf( stdout, "flags: %d\n", (int)r->stats.flags );

    fprintf( stdout, "\n" );
}
