
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit osiris.shmoo.com for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      scale.h
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#ifndef SCALE_H
#define SCALE_H

#include "list.h"
#include "debug.h"
#include "configuration.h"
#include "utilities.h"
#include "string_list.h"
#include "database_handler.h"


#define PROGRAM_NAME       "scale"
#define PROGRAM_VERSION    "1.5.2"

#define DATE_FORMAT_STRING "%b %d, %Y %H:%M"

#ifdef MODULE_MYSQL
#define DB_MODULE "mysql"
#endif

#ifdef MODULE_GDBM
#define DB_MODULE "gdbm"
#endif

/* scale global data. */

configuration *config_module           = NULL;

FILE *output_file                      = NULL;
FILE *config_file                      = NULL;

char config_file_path[MAX_PATH_LEN]    = "";
char output_file_path[MAX_PATH_LEN]    = "";

char left_database_path[MAX_PATH_LEN]  = "";
char right_database_path[MAX_PATH_LEN] = "";

database_handler *left_database        = NULL;
database_handler *right_database       = NULL;

string_list *new_file_list             = NULL;
string_list *missing_file_list         = NULL;

/* comparison stats. */

unsigned long compare_count            = 0;
unsigned long difference_count         = 0;
unsigned long new_record_count         = 0;
unsigned long missing_record_count     = 0;

/* progress meters. */

unsigned long duplicate_progress       = 0;
unsigned long new_progress             = 0;
unsigned long missing_progress         = 0;

/* default behavior. */

bool print_duplicates                  = FALSE;
bool print_new                         = TRUE; 
bool print_missing                     = TRUE; 
bool quiet_mode                        = FALSE;
bool custom_config                     = FALSE;
bool no_prompts                        = FALSE;

/* debug categories. */

unsigned int debug_categories = DEBUG_NONE;


/* function prototypes. */

void initialize( int argument_count, char *argument_list[] );

void process();
void process_with_config();

void halt();
void check_arguments( int argument_count, char *argument_list[] );

void print_usage();
void print_program_info();

void print_database_header( database_handler *db );
void print_database_contents( database_handler *db );
void print_complete_database_contents( database_handler *db );
void print_database_checksums( database_handler *db );
void print_database_config( database_handler *db );
void print_database_error( database_handler *db );
void print_complete_file_record( record *r );

void compare_records( record *left_record, record *right_record );
bool records_differ( record *left_record, record *right_record );

void process_records();
void process_records_with_config();

bool found_config_attributes_for_records( record *left_record,
                                          record *right_record );

void print_new_records();
void print_missing_records();
void print_duplicate_records( database_handler *db );

void open_databases();
void close_databases();

bool open_database( database_handler *db );
void close_database( database_handler *db );

void open_output_file();
void close_output_file();

void print_output_header();
void print_summary();

void print_progress_counter( const char *message, unsigned long counter );
void reset_progress_counter();

void verify_config_file();
void verify_config_state();

#endif

