
/******************************************************************************
**
**  Copyright (C) 2001  - the shmoo group -
**
**  This program is free software; you can redistribute it and/or
**  modify it, however, you cannot sell it.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
**  You should have received a copy of the license attached to the
**  use of this software.  If not, visit www.shmoo.com/osiris for
**  details.
**
******************************************************************************/

/******************************************************************************
**
**    The Shmoo Group (TSG)
**
**    File:      utilities.h
**    Author:    Brian Wotring
**
**    Date:      June 22, 2001.
**    Project:   osiris
**
******************************************************************************/

#ifndef UTILITIES_H
#define UTILITIES_H

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/param.h>

#include "config.h"

typedef int bool;

#ifndef FALSE
#define FALSE   0
#define TRUE    !FALSE
#endif

#ifndef MAX_PATH_LEN
#ifndef MAX_PATH
#define MAX_PATH_LEN ( 1023 + 255 )
#else
#define MAX_PATH_LEN MAX_PATH
#endif
#endif

#ifdef HAVE_U_INT32_T
typedef u_int32_t osi_uint32;
#else
typedef unsigned int osi_uint32;
#endif

#ifdef HAVE_U_INT64_T
typedef u_int64_t osi_uint64;
#else
typedef long long osi_uint64;
#endif

#ifdef WORDS_BIGENDIAN
#define HTONLL(x)   (x)
#define NTOHLL(x)   (x)
#else
#define HTONLL(x)   ( byte_swap64(x) )
#define NTOHLL(x)   ( byte_swap64(x) )
#endif

#define MAXIMUM( first, second )	( ( first > second ) ? first : second )
#define MINIMUM( first, second )	( ( first > second ) ? second : first )

/* 64 bit field conversions. */

osi_uint64 byte_swap64( osi_uint64 value );

/* logging methods. */

void osi_log_message( char *message, ... );
void osi_log_message_exit( char *message, ... );
void osi_log_message_to_file( FILE *file, char *message, ... );

bool osi_continue_prompt( char *message, ... );

/* memory and file operations. */

void osi_free( void *memory );
void *osi_malloc( size_t size );

FILE *osi_fopen( const char *file_name, const char *mode );

bool file_exists( const char *file_path );
bool directory_exists( const char *directory_path );

/* string functions. */

char *trim_white_space( char *string );
char *get_string_up_to_token( char *string, char *result, char token );
char *get_token( char *string, char *token );
void lowercase_string( char *string );

void get_file_attribute_string( char *buffer, int buffer_size,
                                mode_t attributes );

/* safe string wrappers. */

size_t safe_strlcpy( char *dst, const char *src, size_t nchars );
size_t safe_strlcat( char *dst, const char *src, size_t nchars );

size_t safe_vsnprintf( char *str, size_t nchars, const char  *format,
                       va_list ap );

size_t safe_snprintf( char *str, size_t nchars, const char *format, ... );

/* base64 encode/decode methods courtesy of apache. */

int base64_decode( char *buffer, const char *encoded_string );
int base64_decode_binary( unsigned char *buffer, const char *encoded_string );
int base64_encode( char *buffer, const char *string, int string_length );
int base64_encode_binary( char *buffer, const unsigned char *string,
                          int string_length );

/* file escape routine. */

char * escape_filename( char *file_path );

#endif
